/*
 * Copyright (C) 2013  Equinox Software, Inc.
 * Mike Rylander <miker@esilibrary.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

BEGIN;

DROP SCHEMA IF EXISTS depository;
CREATE SCHEMA depository;

CREATE TABLE depository.source_system ( name TEXT PRIMARY KEY );
INSERT INTO depository.source_system (name)
VALUES ('Evergreen-2.1')
      ,('Evergreen-2.2')
      ,('Evergreen-2.3')
      ,('Evergreen-2.4')
;

CREATE TABLE depository.package (
    id          SERIAL  PRIMARY KEY,
    author      INT     NOT NULL REFERENCES actor.usr (id),
    source      TEXT    NOT NULL REFERENCES depository.source_system (name),
    pub         BOOL    NOT NULL DEFAULT TRUE,
    title       TEXT    NOT NULL,
    pkg_version TEXT    NOT NULL,
    created     DATE    NOT NULL DEFAULT NOW(),
    description TEXT,
    image       BYTEA
);

CREATE TABLE depository.package_compat_map (
    id              SERIAL  PRIMARY KEY,
    package         INT     NOT NULL REFERENCES depository.package (id),
    compat_system   TEXT    NOT NULL REFERENCES depository.source_system (name)
);

CREATE TABLE depository.rating (
    id          SERIAL  PRIMARY KEY,
    usr         INT     NOT NULL REFERENCES actor.usr (id),
    package     INT     NOT NULL REFERENCES depository.package (id),
    rating      INT     NOT NULL,
    created     TIMESTAMP   NOT NULL DEFAULT NOW(),
    comments    TEXT
);

CREATE VIEW depository.derived_rating AS
    WITH setting(max_age) AS (
        SELECT  COALESCE(NULLIF(value,'0'),'1')::INT::TEXT AS max_age
          FROM  sharestuff.config_flag
          WHERE name = 'rating.new_rating_bump.days'
                AND enabled
          LIMIT 1
    ), duplicate_list(id,dup_count) AS (
        SELECT  id,
                setting.max_age::INT - DATE_PART('day', NOW() - created )::INT AS dup_count
          FROM  depository.rating, setting
          WHERE created > NOW() - (setting.max_age || ' days')::interval
            UNION
        SELECT  id,
                1 AS dup_count
          FROM  depository.rating, setting
          WHERE created <= NOW() - (setting.max_age || ' days')::interval
    ), sized_arrays(package,rating_array) AS (
        SELECT  package,
                ARRAY_FILL( rating, ARRAY_APPEND(NULL::INT[], duplicate_list.dup_count) ) AS rating_array
          FROM  depository.rating
                JOIN duplicate_list USING (id)
    ), flattened_duplicated_ratings(package,rating) AS (
        SELECT  package, UNNEST( rating_array ) AS rating
          FROM  sized_arrays
    )
    SELECT  p.id AS package, AVG( r.rating ) AS rating
      FROM  depository.package AS p
            LEFT JOIN flattened_duplicated_ratings AS r ON p.id = r.package
      GROUP BY 1;
                

CREATE TABLE depository.base_object_type (
    id      SERIAL  PRIMARY KEY,
    label   TEXT    NOT NULL,
    hint    TEXT    NOT NULL
);

CREATE TABLE depository.versioned_object_type (
    id          SERIAL  PRIMARY KEY,
    base_type   INT     NOT NULL REFERENCES depository.base_object_type (id),
    system      TEXT    NOT NULL REFERENCES depository.source_system (name)
);

CREATE TABLE depository.versioned_object_type_property_ignore (
    id          SERIAL  PRIMARY KEY,
    otype       INT     NOT NULL REFERENCES depository.versioned_object_type (id),
    property    TEXT    NOT NULL
);

CREATE TABLE depository.item (
    id          SERIAL  PRIMARY KEY,
    package     INT     NOT NULL REFERENCES depository.package (id),
    obj_type    INT     NOT NULL REFERENCES depository.versioned_object_type (id),
    data        TEXT    CHECK (sharestuff.is_json(data))
);

CREATE TABLE depository.item_field (
    id          SERIAL  PRIMARY KEY,
    item        INT     REFERENCES depository.item (id),
    container   INT     REFERENCES depository.item_field (id),
    property    TEXT,
    value       TEXT    CHECK (sharestuff.is_json(value)),
    multival    BOOL    NOT NULL DEFAULT FALSE,
    CONSTRAINT  item_or_container CHECK (COALESCE(item,container) IS NOT NULL)
);

COMMIT;

